<?php
/**
 * @package			"VirtueMart Importer Addon for JoomShopping"
 * @version			2.0 [2012-07-02]
 * @compatibility	PHP 5.2/5.3, Joomla 2.5.18, JoomShopping 3.16.2, VirtueMart 2.0.26 
 * @author			MAXXmarketing
 * @copyright		GNU/GPL
*/
 
defined('_JEXEC') or die('Restricted Access');

class IeVMartImporter extends IeController {
	private $DB;
	private $jsLang;
	private $jsConf;

	private $ieTbl;
    private $jsTags;

	# Settings

	# Delete JoomShopping data before import: 1 | 0
	private $deleteJSData;
	
	# Make subproducts(items) in JShop relative to parent: 1 | 0
	private $makeSubproductsRelative;
	# Add VMart subproduct(items) attributes to JShop subproduct title
	private $addAttrsToSubproductTitle;
	# Make all free attributes required
	# (in VMart free attributes are set for each product, so they are all required): 1 | 0
	private $makeFreeAttributesRequired;
	# Which style of attributes to use: jshop | vmart
	private $attrStyle;
	# Add product type title before characteristic label: 1 | 0
	private $characteristicPrefix;
	
	# Which category image to use: thumb | full
	# VMart have full and thumb image for category (but at frontend used only thumb), JShop only one
	private $catImage;

    private $importLanguage;
	# Which vendor logo to use: thumb | full
	# VMart have full and thumb logo for vendors, JShop only one
	private $vendorLogo;
	# Resize images or leave original: 1 | 0
	private $resizeImages;
	
	# Autofill META tags (using product title & description ): 1 | 0
	private $autofillMeta;
	# Generate aliases for category, manufacturer & product using product title: 1 | 0
	private $generateAliases;

	# Relations between VMart items ids and inserted to JShop: vmId => jsId
	# I need store them to survive relations from VMart DB to JShop
	private $vmIdXjsId;
	
	# I need complex query to get product price and currency: vmProdId => currCode
	# Last I need in several places, so I store it globally
	private $vmProductsCurrencies;

    private $vmConfig;
    
	# Constructor
	public function __construct($ieId) {
		# Set infinite max_execution_time
		@set_time_limit(0);
		
		# Set error reporting
		error_reporting(E_ALL ^ E_NOTICE);
		
        $document = JFactory::getDocument();
        $document->addStyleSheet(JURI::root(true).'/administrator/components/com_jshopping/importexport/vmartimporter/style.css');
        
		# Set timezone
		if (!ini_get('date.timezone'))
			@date_default_timezone_set(date_default_timezone_get());

		$this->DB = JFactory::getDBO();
		$this->jsLang = JSFactory::getLang();
		$this->jsConf = JSFactory::getConfig();

		# Thumnail processing class
		include_once ($this->jsConf->path.'lib'.'/'.'image.lib.php');
		
		# This lib is included from different dirs, so include_once don't work
		# It can happens when automatic execution is on simultaneously for different imports and exports
		# which are using this lib.
		if (!class_exists('MMLib')) include (dirname(__FILE__).'/'.'mm.lib.php');

		# IE tbl
		JTable::addIncludePath(JPATH_COMPONENT_SITE.'/'.'tables');
		
		$this->ieTbl = JTable::getInstance('ImportExport', 'jshop');

        $this->ieTbl->load($ieId);
        
		# Getting path to VMart images
		
		# Support for legacy mode
		$mosConfig_absolute_path = $GLOBALS['mosConfig_absolute_path'] = JPATH_SITE;
        
		# Init settings
		$this->deleteJSData = 1;
		
		$this->makeSubproductsRelative = 1;
		$this->addAttrsToSubproductTitle = 1;
		$this->makeFreeAttributesRequired = 1;
		$this->attrStyle = 'vmart';
		$this->characteristicPrefix = 0;
		
		$this->catImage = 'thumb';
        $this->importLanguage = 'auto';
		$this->vendorLogo = 'full';
		$this->resizeImages = 1;
		
		$this->autofillMeta = 1;
		$this->generateAliases = 1;

		# Load settings from DB
		$params = parseParamsToArray($this->ieTbl->params);
        
		MMLib::setIfValNoEmpty($this->deleteJSData, $params['deleteJSData']);
		
		MMLib::setIfValNoEmpty($this->makeSubproductsRelative, $params['makeSubproductsRelative']);
		MMLib::setIfValNoEmpty($this->addAttrsToSubproductTitle, $params['addAttrsToSubproductTitle']);
		MMLib::setIfValNoEmpty($this->makeFreeAttributesRequired, $params['makeFreeAttributesRequired']);
		MMLib::setIfValNoEmpty($this->attrStyle, $params['attrStyle']);
		MMLib::setIfValNoEmpty($this->characteristicPrefix, $params['characteristicPrefix']);
		
		MMLib::setIfValNoEmpty($this->catImage, $params['catImage']);
        MMLib::setIfValNoEmpty($this->importLanguage, $params['importLanguage']);  
		MMLib::setIfValNoEmpty($this->vendorLogo, $params['vendorLogo']);
		MMLib::setIfValNoEmpty($this->resizeImages, $params['resizeImages']);
		
		MMLib::setIfValNoEmpty($this->autofillMeta, $params['autofillMeta']);
		MMLib::setIfValNoEmpty($this->generateAliases, $params['generateAliases']);

		# For what I need to store relations
		$this->vmIdXjsId = array('category',
								 'manufacturer',
								 'product',
								 'tax',
								 'usergroup',
								 'vendor'
		);

		$this->vmProductsCurrencies = array();
   		if ($this->_checkForJS() && $this->_checkForVM()){
        	$this->vmConfig = $this->getVmConfig();
            
        	if (!isset($this->vmConfig['active_languages'])){
                if (isset($this->vmConfig['vmlang']) && !empty($this->vmConfig['vmlang'])){
                    $tagArray = explode('_', $this->vmConfig['vmlang']);
                    $this->vmConfig['active_languages'] = array($tagArray[0]."-".strtoupper($tagArray[1]));
                } else {
                    $this->vmConfig['active_languages'] = array();
                }
        		
        	}
   		} else {
   			$this->vmConfig['active_languages'] = array();
   		}

        $query = "SELECT language FROM `#__jshopping_languages`";
        $this->DB->setQuery($query);
        $this->jsTags = $this->DB->loadResultArray();
		parent::__construct();
	}

	# Check for JShop
	private function _checkForJS() {
		# Check for the existence of JShop component
		define ('JSHOP_PATH', JPATH_ADMINISTRATOR.'/'.'components'.'/'.'com_jshopping');
		
		if(!is_dir(JSHOP_PATH))
			return FALSE;
		
		# Check for the existence of JShop tables (enough 1 tbl)
		$this->DB->setQuery("SHOW TABLES LIKE '%jshopping_categories%'");
		$this->DB->query();

		if ($this->DB->getNumRows() == 0)
			return FALSE;
		
		return TRUE;
	}
	
	# Check for VMart
	private function _checkForVM() {
		# Check for the existence of JShop component
		define ('VMART_PATH', JPATH_ADMINISTRATOR.'/'.'components'.'/'.'com_virtuemart');
		
		if(!is_dir(VMART_PATH))
			return FALSE;
		
		# Check for the existence of VMart tables (enough 1 tbl)
		$this->DB->setQuery("SHOW TABLES LIKE '%virtuemart_categories%'");
		$this->DB->query();

		if ($this->DB->getNumRows() == 0)
			return FALSE;
		
		return TRUE;
	}

	# Truncate tables
	private function _deleteJSData() {
		# ========== Delete files ==========
		
		# Attributes images
		$query = "SELECT image
				  FROM `#__jshopping_attr_values`";
		$this->DB->setQuery($query);
		$this->DB->query();
		
		$res = $this->DB->loadResultArray();
		
		if (!empty($res[0]))
			foreach ($res as $currFile)
				if (!empty($currFile))
					@unlink($this->jsConf->image_attributes_path.'/'.$currFile);
		
		
		# Categories images
		$query = "SELECT category_image
				  FROM `#__jshopping_categories`";
		$this->DB->setQuery($query);
		$this->DB->query();
		
		$res = $this->DB->loadResultArray();
		
		if (!empty($res[0]))
			foreach ($res as $currFile)
				if (!empty($currFile))
					@unlink($this->jsConf->image_category_path.'/'.$currFile);
			
		# Manufacturers logos
		$query = "SELECT manufacturer_logo
				  FROM `#__jshopping_manufacturers`";
		$this->DB->setQuery($query);
		$this->DB->query();
		
		$res = $this->DB->loadResultArray();
		
		if (!empty($res[0]))
			foreach ($res as $currFile)
				if (!empty($currFile))
					@unlink($this->jsConf->image_manufs_path.'/'.$currFile);			
		
		# Vendors logos
		# in DB they are urls and can be stored not in jsConf->image_vendors_path, so I juct clean this path
		MMLib::cleanDir($this->jsConf->image_vendors_path, array('noimage.gif'));

		# Products files (both sale- and demo)
		$query = "SELECT file, demo
				  FROM `#__jshopping_products_files`";
		$this->DB->setQuery($query);
		$this->DB->query();
		
		foreach ($this->DB->loadObjectList() as $currObj) {
			if (!empty($currObj->file))
				@unlink($this->jsConf->files_product_path.'/'.$currObj->file);
			
			if (!empty($currObj->demo))
				@unlink($this->jsConf->demo_product_path.'/'.$currObj->demo);
		}
		
		# Products images
		$query = "SELECT image_thumb AS img_thumb_name,
					  image_name AS img_medium_name,
					  image_full AS img_full_name
				  FROM `#__jshopping_products_images`";
		$this->DB->setQuery($query);
		$this->DB->query();
		
		foreach ($this->DB->loadObjectList() as $currObj)
			if (!empty($currObj->img_full_name)) {
				@unlink($this->jsConf->image_product_path.'/'.$currObj->img_thumb_name);
				@unlink($this->jsConf->image_product_path.'/'.$currObj->img_medium_name);
				@unlink($this->jsConf->image_product_path.'/'.$currObj->img_full_name);
			}
		
		
		# Products videos
		$query = "SELECT video_name
				  FROM `#__jshopping_products_videos`";
		$this->DB->setQuery($query);
		$this->DB->query();
		
		$res = $this->DB->loadResultArray();
		
		if (!empty($res[0]))
			foreach ($res as $currFile)
				if (!empty($currFile))
					@unlink($this->jsConf->video_product_path.'/'.$currFile);
		
		
		# ========== Clean DB ==========
		
		# JShop adds characteristics ids to products table, delete them
		$this->DB->setQuery("SELECT `id` FROM `#__jshopping_products_extra_fields`");
		$this->DB->query();
		
		$characteristicsIds = $this->DB->loadResultArray();
		
		foreach ($characteristicsIds as $characteristicId) {
			$this->DB->setQuery("ALTER TABLE `#__jshopping_products`
								DROP COLUMN `extra_field_$characteristicId`");
			$this->DB->query();
		}
		
		# JShop adds attributes ids fields to products_attr table, delete them
		# (only attributes that uses JShop style)
		$this->DB->setQuery("SELECT `attr_id` FROM `#__jshopping_attr`");
		$this->DB->query();
		
		$attrIds = $this->DB->loadResultArray();
		foreach ($attrIds as $attrId) {
			$this->DB->setQuery("SHOW COLUMNS FROM `#__jshopping_products_attr` LIKE 'attr_$attrId'");
			if (count($this->DB->loadObject()) > 0){
				$this->DB->setQuery("ALTER TABLE `#__jshopping_products_attr` DROP COLUMN `attr_$attrId`");
				$this->DB->query();
			}
		}

		# Truncate tables in which I will import
		$usedTables = array('#__jshopping_attr',
							'#__jshopping_attr_values',
							'#__jshopping_categories',
							'#__jshopping_coupons',
							'#__jshopping_free_attr',
							'#__jshopping_manufacturers',
							'#__jshopping_products',
							'#__jshopping_products_attr',
							'#__jshopping_products_attr2',
							'#__jshopping_products_extra_fields',
							'#__jshopping_products_extra_field_values',
							'#__jshopping_products_files',
							'#__jshopping_products_free_attr',
							'#__jshopping_products_images',
							'#__jshopping_products_prices',
							'#__jshopping_products_relations',
							'#__jshopping_products_reviews',
							'#__jshopping_products_videos',
							'#__jshopping_products_to_categories',
							'#__jshopping_shipping_method',
							'#__jshopping_shipping_method_price',
							'#__jshopping_shipping_method_price_countries',
							'#__jshopping_shipping_method_price_weight',
							'#__jshopping_taxes',
							'#__jshopping_users',
							'#__jshopping_vendors');
		
		foreach ($usedTables as $tblName) {
			$this->DB->setQuery('TRUNCATE TABLE `'.$tblName.'`');
			$this->DB->query();
		}
		
		# User Groups tbl should contain default group, so I can't just truncate it
		$this->DB->setQuery("DELETE FROM `#__jshopping_usergroups` WHERE usergroup_id  <> 1");
		$this->DB->query();

		# Meaningless tables after import of new data
		$meaninglessTables = array('#__jshopping_cart_temp');

		foreach ($meaninglessTables as $tblName) {
			$this->DB->setQuery('TRUNCATE TABLE `'.$tblName.'`');
			$this->DB->query();
		}
	}
	
    private function _convertDiscountPrice($vmPrice, $vmCurrencyId) {
        $query = "SELECT currency_value
                  FROM #__jshopping_currencies
                  WHERE currency_code_iso = '(SELECT currency_code_3
                  FROM #__virtuemart_currencies
                  WHERE virtuemart_currency_id = '$vmCurrencyId' LIMIT 1)'";
    
        $this->DB->setQuery($query);
        $this->DB->query();
        
        return $vmPrice * $this->DB->loadResult();
    }
    
	# Get JShop country ID by VMart country code
	private function _getJSCountryID($vmCountryId) {
        if ($vmCountryId){
            # In VMart field contains code, in JShop - foreign key to countries table record
            $query = "SELECT js_cnt.`country_id`
                      FROM `#__virtuemart_countries` AS vm_cnt 
                      JOIN `#__jshopping_countries` AS js_cnt
                      ON vm_cnt.`virtuemart_country_id` = $vmCountryId AND js_cnt.`country_code` = vm_cnt.`country_3_code` AND js_cnt.`country_code_2` = vm_cnt.`country_2_code`";

            $this->DB->setQuery($query);
            return $this->DB->loadResult();
        }
        return 0;
	}

	# Get state by country ID and state code
	private function _getVMStateName($vmStateId) {
        if ($vmStateId){
            $query = "SELECT `state_name` FROM `#__virtuemart_states` WHERE `virtuemart_state_id` = $vmStateId LIMIT 1";
            $this->DB->setQuery($query);

            return $this->DB->loadResult();
        }
        return '';
	}
	
	# Convert price using JShop prices configuration
	private function _convertPrice($mathOp, $vmProdPrice, $vmDiscPrice, $vmProdCurr, $vmDiscCurr, $discountActual) {
		$price = 0;
        $query = "SELECT currency_value, currency_code_iso
				  FROM `#__jshopping_currencies`
				  WHERE currency_code_iso IN ('$vmProdCurr', '$vmDiscCurr')";
        $this->DB->setQuery($query);   
        
        $currArray = array();
	    foreach ($this->DB->loadObjectList() as $value)
            $currArray[$value->currency_code_iso] = $value->currency_value;
            
        $prodCurr = array($vmProdCurr, $vmDiscCurr);  

        foreach($prodCurr as $value){
        	if (!array_key_exists($value, $currArray) && !empty($value)){  
        		$query = "SELECT MAX(currency_ordering) FROM `#__jshopping_currencies`";
        		$this->DB->setQuery($query);
        		$ord = $this->DB->loadResult();
        		
        		$query = "INSERT INTO `#__jshopping_currencies` (`currency_name`, `currency_code`, `currency_code_iso`, `currency_ordering`, `currency_value`, `currency_publish`) 
        			  VALUES ('$value', '$value', '$value', $ord, '1', '1')";
        		$this->DB->setQuery($query);
        		$this->DB->query();
        		
        		$currArray[$value] = 1;
       	 	}   
        }
        if ($discountActual && !empty($mathOp) && !empty($vmDiscPrice) && !empty($vmDiscCurr)){
        	switch ($mathOp){
                case '-' :
                    $price = ($vmProdPrice * $currArray[$vmProdCurr]) - ($vmDiscPrice * $currArray[$vmDiscCurr]);
                break;
                case '+' :
                    $price = ($vmProdPrice * $currArray[$vmProdCurr]) + ($vmDiscPrice * $currArray[$vmDiscCurr]);
                break;     
            }              
        }else{
            $price = $vmProdPrice * $currArray[$vmProdCurr];     
        }    
		return $price;
	}
	
	# Generate alias
	private function _generateAlias($fieldName, $forWhat, $title) {
		# Reserved aliases
		static $resAliases;
		
		if (!is_array($resAliases)){
			jimport('joomla.filesystem.folder');
	
			$files = JFolder::files(JPATH_ROOT."/components/com_jshopping/controllers");
			$resAliases = array();
			
			foreach($files as $file)
				$resAliases[] = str_replace(".php", "", $file);
		}
		
		# Generate alias
		$alias = $title;
		
		$alias = str_replace(" ", "-", $alias);
		$alias = (string) preg_replace('/[\x00-\x1F\x7F<>"\'$#%&\?\/\.\)\(\{\}\+\=\[\]\\\,:;]/', '', $alias);
		$alias = JString::strtolower($alias);
		
		# Escaped alias
		$escAlias = $this->DB->getEscaped($alias);
		
		
		# Checking for existing alias
		switch ($forWhat) {
			case 'category':
				$query = "SELECT category_id FROM #__jshopping_categories WHERE `$fieldName` = '$escAlias' LIMIT 1";
				
				$this->DB->setQuery($query);
				$this->DB->query($query);
				
				$res = $this->DB->loadResult();
				
				if ($res || in_array($alias, $resAliases))
					return '';

				break;
			case 'manufacturer':
				$query = "SELECT manufacturer_id FROM #__jshopping_manufacturers WHERE `$fieldName` = '$escAlias' LIMIT 1";
				
				$this->DB->setQuery($query);
				$this->DB->query($query);
				
				$res = $this->DB->loadResult();

				if ($res || in_array($alias, $resAliases))
					return '';
					
				break;
			case 'product':
				$query = "SELECT product_id FROM #__jshopping_products WHERE `$fieldName` = '$escAlias' LIMIT 1";
				
				$this->DB->setQuery($query);
				$this->DB->query($query);
				
				if ($this->DB->loadResult())
					return '';
				
				break;
		}
		
		return $alias;
	}
	
	# Get VMart subproduct attributes string
	private function _getSubproductAttrsStr($subProdId) {
		$query = "SELECT prod_attr.`custom_value` AS attribute_value, prod_attr.`virtuemart_custom_id`, vm_customs.`custom_title` AS attribute_name
        		  FROM `#__virtuemart_product_customfields` AS prod_attr
                  LEFT JOIN `#__virtuemart_customs` as vm_customs
                  USING (virtuemart_custom_id)
				  WHERE prod_attr.virtuemart_product_id = $subProdId";
		
		$this->DB->setQuery($query);
		
		$adjStr = '';
		
		$attrsList = $this->DB->loadObjectList();

		if (!empty($attrsList[0])) {
			$adjStr .= '(';
			
			for ($i = 0; $i < count($attrsList); $i++) {
				$adjStr .= $attrsList[$i]->attribute_name.': '.$attrsList[$i]->attribute_value;
				
				if ($i != (count($attrsList) - 1))
					$adjStr .= ', ';
			}
			
			$adjStr .= ')';
		} 
		return $adjStr;
	}
	
	# Get last ordering from DB
	private function _getLastOrdering($forWhat) {
		if ($this->deleteJSData)
			return 0;
		
		switch ($forWhat) {
			case 'attributes':
				$query = "SELECT MAX(attr_ordering) FROM #__jshopping_attr";
				break;
			case 'categories':
				$query = "SELECT MAX(ordering) FROM #__jshopping_categories";
				break;
			case 'characteristics':
				$query = "SELECT MAX(ordering) FROM #__jshopping_products_extra_fields";
				break;
			case 'free_attributes':
				$query = "SELECT MAX(ordering) FROM #__jshopping_free_attr";
				break;
			case 'manufacturers':
				$query = "SELECT MAX(ordering) FROM #__jshopping_manufacturers";
				break;
			case 'shipping_methods':
				$query = "SELECT MAX(ordering) FROM #__jshopping_shipping_method";
				break;
			default:
				return FALSE;
				
		}
		
		$this->DB->setQuery($query);
		$this->DB->query();
				
		return $this->DB->loadResult();
	}

	# Copy (and resize if set) image for category
	private function _processCategoryImage($imgName,  $imgUrl) {
        # Get first available filename if file with given filename already exists
		$availFilename = MMLib::getAvailableFileName($this->jsConf->image_category_path.'/'.$imgName);

		if ($this->resizeImages)
			# Copy resized
			ImageLib::resizeImageMagic(JPATH_SITE.'/'.$imgUrl,
				$this->jsConf->image_category_width,
				$this->jsConf->image_category_height,
				$this->jsConf->image_cut,
				$this->jsConf->image_fill,
				$this->jsConf->image_category_path.'/'.$availFilename,
				$this->jsConf->image_quality,
				$this->jsConf->image_fill_color);
		else 
			# Just copy
			copy(JPATH_SITE.'/'.$imgUrl, $this->jsConf->image_category_path.'/'.$availFilename);

		chmod($this->jsConf->image_category_path.'/'.$availFilename, 0777);

		# Return available filename to set right DB field
		return $availFilename;
	}

    private function _processManufacturerImage($imgName, $imgUrl, $imgUrlThumb) {
        # Get first available filename if file with given filename already exists
        $availFilename = MMLib::getAvailableFileName($this->jsConf->image_manufs_path.'/'.$imgName);
        # Just copy
        if (file_exists(JPATH_SITE.'/'.$imgUrlThumb)){
            copy(JPATH_SITE.'/'.$imgUrlThumb, $this->jsConf->image_manufs_path.'/'.$availFilename);
        }
        else 
            copy(JPATH_SITE.'/'.$imgUrl, $this->jsConf->image_manufs_path.'/'.$availFilename);

        chmod($this->jsConf->image_manufs_path.'/'.$availFilename, 0777);
        
        # Return available filename to set right DB field
        return $availFilename;
    }
    
	# Copy (and resize if set) full-size, medium- and thumb-images for product
	private function _processProductImages($imgsInfo, $prodId, $resizeThumb = FALSE) {
		# Full
		# Get first available filename if file with given filename already exists
		$availFilenameFull = MMLib::getAvailableFileName($this->jsConf->image_product_path.'/'.
														 'full_'.$imgsInfo->ImgBasename);										 
		copy($imgsInfo->srcFullImgPath, $this->jsConf->image_product_path.'/'.$availFilenameFull);
		chmod($this->jsConf->image_product_path.'/'.$availFilenameFull, 0777);

		# Medium
		# Get first available filename if file with given filename already exists
		$availFilenameMedium = MMLib::getAvailableFileName($this->jsConf->image_product_path.'/'.
														   $imgsInfo->ImgBasename);										 
		if ($this->resizeImages)
			# Copy resized
			ImageLib::resizeImageMagic($imgsInfo->srcFullImgPath,
				$this->jsConf->image_product_full_width,
				$this->jsConf->image_product_full_height,
				$this->jsConf->image_cut,
				$this->jsConf->image_fill,
				$this->jsConf->image_product_path.'/'.$availFilenameMedium,
				$this->jsConf->image_quality,
				$this->jsConf->image_fill_color);
		else 
			# Just copy
			copy($imgsInfo->srcFullImgPath, $this->jsConf->image_product_path.'/'.$availFilenameMedium);

		chmod($this->jsConf->image_product_path.'/'.$availFilenameMedium, 0777);

		# Thumb
		# Get first available filename if file with given filename already exists
		$availFilenameThumb = MMLib::getAvailableFileName($this->jsConf->image_product_path.'/'.
														  'thumb_'.$imgsInfo->ImgBasename);
		if (($this->resizeImages || $resizeThumb) && isset($imgsInfo->srcThumbImgPath))
			# Copy resized
			ImageLib::resizeImageMagic($imgsInfo->srcThumbImgPath,
				$this->jsConf->image_product_width,
				$this->jsConf->image_product_height,
				$this->jsConf->image_cut,
				$this->jsConf->image_fill,
				$this->jsConf->image_product_path.'/'.$availFilenameThumb,
				$this->jsConf->image_quality,
				$this->jsConf->image_fill_color);
		else if (!isset($imgsInfo->srcThumbImgPath)){
			ImageLib::resizeImageMagic($imgsInfo->srcFullImgPath,
				$this->jsConf->image_product_width,
				$this->jsConf->image_product_height,
				$this->jsConf->image_cut,
				$this->jsConf->image_fill,
				$this->jsConf->image_product_path.'/'.$availFilenameThumb,
				$this->jsConf->image_quality,
				$this->jsConf->image_fill_color);	
		} else
			# Just copy
			copy($imgsInfo->srcThumbImgPath, $this->jsConf->image_product_path.'/'.$availFilenameThumb);

		chmod($this->jsConf->image_product_path.'/'.$availFilenameThumb, 0777);

		# DB import
		# Update product tbl
		$jsProdTbl =& JTable::getInstance('Product', 'jshop');
		$jsProdTbl->load($prodId);
		
        if (empty($jsProdTbl->product_full_image)) {
            $jsProdTbl->set('product_full_image', $availFilenameFull);
		    $jsProdTbl->set('product_name_image', $availFilenameMedium);
		    $jsProdTbl->set('product_thumb_image', $availFilenameThumb);
        }
		
		$jsProdTbl->store();

		# Insert to product images tbl
		$jsImgTbl =& JTable::getInstance('Image', 'jshop');

		$jsImgTbl->set('product_id', $prodId);
		$jsImgTbl->set('image_full', $availFilenameFull);
		$jsImgTbl->set('image_name', $availFilenameMedium);
		$jsImgTbl->set('image_thumb', $availFilenameThumb);

		$jsImgTbl->store();
	}
	
    # Print what processing and flush
	private function _printImportingHeader($subj) {
		echo _JSHOP_IE_VMIMPORT_IMPORTING.' <b>'.$subj.'</b>... ';
		flush();
		@ob_flush();
	}

	# Print processed footer
	private function _printImportingFooter() {
		echo '<font color="green">'._JSHOP_IE_VMIMPORT_SUCCEEDED.'</font></b><br>';
		flush();
		@ob_flush();
	}

	# Attributes
	private function _importAttributes() {
		# Get last ordering
		$ordCnt = $this->_getLastOrdering('attributes');
		$query = "SELECT vm_cust.*
				  FROM `#__virtuemart_customs` AS vm_cust 
                  WHERE vm_cust.`is_cart_attribute` = '1' 
                  AND vm_cust.`field_type` <> 'E' AND vm_cust.`field_type` <> 'X' AND vm_cust.`field_type` <> 'P' AND vm_cust.`field_type` <> 'D' AND vm_cust.`field_type` <> 'A' AND vm_cust.`field_type` <> 'Y'";

		$this->DB->setQuery($query);
		foreach ($this->DB->loadObjectList() as $vmProd) {
            # Create new attr DB entry
		    $jsAttrTbl = JTable::getInstance('Attribut', 'jshop');
		    $jsAttrTbl->set('attr_ordering', ++$ordCnt);
		    $jsAttrTbl->set('cats', 'a:0:{}'); // JShop makes so 
		    $jsAttrTbl->set('attr_type', 1); // Default
            $jsAttrTbl->set('allcats', 1); // Default  
		    $jsAttrTbl->set($this->jsLang->get('name'), $vmProd->custom_title);
		    $jsAttrTbl->set('independent', ($this->attrStyle == 'jshop') ? 0 : 1); // 0 - JShop style, 1- VMart style
		    $jsAttrTbl->store();
            $this->vmIdXjsId['attributes'][$vmProd->virtuemart_custom_id] = $jsAttrTbl->attr_id;
		}
	}
	
    private function _importProductAttributes() {
        $query = "SELECT vm_prod_cust.`virtuemart_custom_id`, vm_prod_cust.`virtuemart_product_id`, vm_prod_cust.`custom_price`,
                  vm_prod_cust.`custom_value`
                  FROM `#__virtuemart_product_customfields` AS vm_prod_cust
                  LEFT JOIN `#__virtuemart_customs` AS vm_cust
                  ON vm_cust.`virtuemart_custom_id` = vm_prod_cust.`virtuemart_custom_id`
                  WHERE vm_cust.`is_cart_attribute` = '1'
                  AND vm_cust.`field_type` <> 'E' AND vm_cust.`field_type` <> 'X' AND vm_cust.`field_type` <> 'P' AND vm_cust.`field_type` <> 'D' AND vm_cust.`field_type` <> 'A' AND vm_cust.`field_type` <> 'Y'";

        $this->DB->setQuery($query);

        foreach ($this->DB->loadObjectList() as $vmProd) {
            # Create new attr val DB entry
            $jsAttrValTbl = JTable::getInstance('AttributValue', 'jshop');
            $jsAttrValTbl->set('attr_id', $this->vmIdXjsId['attributes'][$vmProd->virtuemart_custom_id]);

            $jsAttrValTbl->set('value_ordering', 1);

            $jsAttrValTbl->set($this->jsLang->get('name'), $vmProd->custom_value);
            $jsAttrValTbl->store();
            $this->vmIdXjsId['attributesValue'][$vmProd->virtuemart_custom_id] = $jsAttrValTbl->value_id;
            
            switch ($this->attrStyle) {
                case 'jshop':
                    # Insert new attributes fields to jshopping_products_attr table
                    
            		$this->DB->setQuery("SHOW COLUMNS FROM `#__jshopping_products_attr` LIKE 'attr_". $this->vmIdXjsId['attributes'][$vmProd->virtuemart_custom_id] ."'");
					if (count($this->DB->loadObject()) <= 0){
						$this->DB->setQuery("ALTER TABLE `#__jshopping_products_attr` ADD `attr_". $this->vmIdXjsId['attributes'][$vmProd->virtuemart_custom_id] ."` INT(11)");
						$this->DB->query();
					
					}
					
                    $jsProdTbl = JTable::getInstance('Product', 'jshop');
                    $jsProdTbl->load($this->vmIdXjsId['product'][$vmProd->virtuemart_product_id]);
                    $jsProdAttrTbl = JTable::getInstance('ProductAttribut', 'jshop');
                    $jsProdAttrTbl->set('product_id', $this->vmIdXjsId['product'][$vmProd->virtuemart_product_id]);
                    
                    # Count price
                    $prodPrice = $jsProdTbl->product_price + $vmProd->custom_price;
                  
                    $jsProdAttrTbl->set('price', $prodPrice);
                    
                    $jsProdAttrTbl->set('count', $jsProdTbl->product_quantity);
                    $jsProdAttrTbl->set('ean', $jsProdTbl->product_ean);
                    $jsProdAttrTbl->set('weight_volume_units', $jsProdTbl->weight_volume_units);
                    $jsProdAttrTbl->set('weight', $jsProdTbl->product_weight);
                     
                    # Set selected values for each attribute

                    $jsProdAttrTbl->set('attr_'.$this->vmIdXjsId['attributes'][$vmProd->virtuemart_custom_id], $this->vmIdXjsId['attributesValue'][$vmProd->virtuemart_custom_id]);
                    
                    $jsProdAttrTbl->store();

                    break;
                case 'vmart':
                        $jsProdAttr2Tbl = JTable::getInstance('ProductAttribut2', 'jshop');
                        
                        $jsProdAttr2Tbl->set('product_id', $this->vmIdXjsId['product'][$vmProd->virtuemart_product_id]);
                        $jsProdAttr2Tbl->set('attr_id', $this->vmIdXjsId['attributes'][$vmProd->virtuemart_custom_id]);
                        $jsProdAttr2Tbl->set('attr_value_id', $this->vmIdXjsId['attributesValue'][$vmProd->virtuemart_custom_id]);
                        $jsProdAttr2Tbl->set('price_mod', '+'); //default
                        $jsProdAttr2Tbl->set('addprice', $vmProd->custom_price);

                        $jsProdAttr2Tbl->store();
                    break;
            }
        }

        $this->DB->setQuery("SELECT GROUP_CONCAT(js_attr.`attr_id`)
                 FROM `#__jshopping_attr_values` AS js_attr_val
                 RIGHT JOIN `#__jshopping_attr` AS js_attr
                 ON (js_attr_val.`attr_id` = js_attr.`attr_id`)
                 WHERE js_attr_val.`attr_id` IS NULL");
        if (count($this->DB->loadResult()) > 0){
        	$ids = $this->DB->loadResult();
        	$query = "DELETE FROM `#__jshopping_attr` WHERE attr_id IN ($ids)";
        	$this->DB->setQuery($query);
        	$this->DB->query();
        }
    }
    
    private function getVmConfig(){
        $query = "SELECT `config` FROM `#__virtuemart_configs` LIMIT 1";
        $this->DB->setQuery($query);
        $jpConfig = $this->DB->loadResult();
        $i = 0;
        $pair = array();
        if (!empty($jpConfig)) {
            $config = explode('|', $jpConfig);
            foreach($config as $item){
                $item = explode('=',$item);
                if(!empty($item[1])){
                    if($item[0]!=='offline_message' ){
                        $pair[$item[0]] = unserialize($item[1] );
                    } else {
                        $pair[$item[0]] = unserialize(base64_decode($item[1]) );
                    }
                } else {
                    $pair[$item[0]] ='';
                }
            }  
        }
        return $pair;
    }
    
    private function _parseVmParams($params){
        $pair = array();
        if (!empty($params)) {
            $config = explode('|', $params);
            foreach($config as $item){
                $item = explode('=',$item);
                if ($item[1][0] == '['){ 
                    $pair[$item[0]] = explode('","', str_replace('"]', '', str_replace('["', '', $item[1]))); 
                }
                else {
                    $value = explode('"', $item[1]); 
                    $pair[$item[0]] = $value[1];   
                }  
            }  
        }
        return $pair;    
    }
    
	# Categories
	private function _importCategories() {
		# Get last ordering
		$ordCnt = $this->_getLastOrdering('categories'); // In VMart ordering can be 0
		$jsCatIdXvmSubcatId = array();
   
        $adv_join = ""; $adv_query = "";   
        foreach ($this->vmConfig['active_languages'] as $tag){
            $vmTag = strtolower(str_replace('-', '_', $tag)); 
            $adv_join .= ' LEFT JOIN `#__virtuemart_categories_'. $vmTag .'` AS vm_cat_desc_'. $vmTag .'
                  ON vm_cat.`virtuemart_category_id` = vm_cat_desc_'. $vmTag .'.`virtuemart_category_id`'; 
                  
            $adv_query .= ' vm_cat_desc_'. $vmTag .'.`metadesc` AS metadesc_'. $vmTag .', 
                            vm_cat_desc_'. $vmTag .'.`metakey` AS metakey_'. $vmTag .', 
                            vm_cat_desc_'. $vmTag .'.`customtitle` AS customtitle_'. $vmTag .', 
                            vm_cat_desc_'. $vmTag .'.`category_name` AS cat_name_'. $vmTag .', 
                            vm_cat_desc_'. $vmTag .'.`category_description` AS cat_desc_'. $vmTag .',
                            vm_cat_desc_'. $vmTag .'.`slug` AS cat_alias_'. $vmTag .', '; 
        }
		$query = "SELECT vm_cat.*, vm_cat_xref.category_parent_id, $adv_query vm_medias.`file_title`, vm_medias.`file_url`, vm_medias.`file_url_thumb`
				  FROM `#__virtuemart_categories` AS vm_cat
				  LEFT JOIN `#__virtuemart_category_categories` AS vm_cat_xref 
                  ON vm_cat.`virtuemart_category_id` = vm_cat_xref.`category_child_id`
                  $adv_join
                  LEFT JOIN `#__virtuemart_category_medias` AS vm_cat_media
                  ON vm_cat.`virtuemart_category_id` = vm_cat_media.`virtuemart_category_id`
                  LEFT JOIN `#__virtuemart_medias` AS vm_medias
                  ON vm_medias.`virtuemart_media_id` = vm_cat_media.`virtuemart_media_id`
                  AND vm_medias.`file_type` = 'category' AND vm_medias.`file_mimetype` LIKE 'image/%'
                  GROUP BY vm_cat.`virtuemart_category_id` ";

		$this->DB->setQuery($query);
		$this->DB->query();

		if (count($this->DB->loadObjectList()))
		foreach ($this->DB->loadObjectList() as $vmCat) {
			# DB import
			$jsCatTbl = JTable::getInstance('Category', 'jshop');

			# Image
			switch ($this->catImage) {
				case 'full':
					if (!empty($vmCat->file_title) && file_exists(JPATH_SITE.'/'.$vmCat->file_url)) {
						$createdFilename = $this->_processCategoryImage($vmCat->file_title, $vmCat->file_url);
						
						$jsCatTbl->set('category_image', $createdFilename);
					}
					break;
				case 'thumb':
					if (!empty($vmCat->file_title) && is_file(JPATH_SITE.'/'.$vmCat->file_url_thumb)){
						$createdFilename = $this->_processCategoryImage($vmCat->file_title, $vmCat->file_url_thumb);
                    }elseif (!empty($vmCat->file_title)) {
                        $createdFilename = $this->_processCategoryImage($vmCat->file_title, $vmCat->file_url);
                    }
                    
                    $jsCatTbl->set('category_image', $createdFilename);
					break;
			}
	
			# Subcat i will assign later, i need first all JShop cats real IDs, because i don't now yet if
			# $vmCat->category_parent_id is already in $this->vmIdXjsId['category'] array
			
			$jsCatTbl->set('category_publish', $vmCat->published);
			$jsCatTbl->set('category_ordertype', 1); // Default
			$jsCatTbl->set('category_template', 'default'); // Default
			
			# If this isn't child category I should leave existing cats at the start of list and save VMart cats ordering,
			# if this is child category ordering duplicate with main categories no matters
			if (!$vmCat->category_parent_id)
				$jsCatTbl->set('ordering', $ordCnt + $vmCat->ordering);
			else
				$jsCatTbl->set('ordering', $vmCat->ordering);

			$jsCatTbl->set('category_add_date', date('Y-m-d H:i:s', $vmCat->created_on));
			$jsCatTbl->set('products_page', $this->jsConf->count_products_to_page); // Config
			$jsCatTbl->set('products_row', $vmCat->products_per_row);
			 
            foreach ($this->vmConfig['active_languages'] as $tag){
                if (in_array($tag, $this->jsTags)){
                    $vmTag = strtolower(str_replace('-', '_', $tag)); 
                    $catName = 'cat_name_'.$vmTag;
                    $catDesc = 'cat_desc_'.$vmTag;
                	$catAlias = 'cat_alias_'.$vmTag;
                    $metadesc = 'metadesc_'.$vmTag;
                    $metakey = 'metakey_'.$vmTag;
                	$customtitle = 'customtitle_'.$vmTag;
                    $jsCatTbl->set('name_'.$tag, $vmCat->$catName);  
                    
                    # Generate alias if such option is checked
                    if ($this->generateAliases || !$this->deleteJSData)
                        $jsCatTbl->set('alias_'.$tag, $this->_generateAlias('alias_'.$tag, 'category', $vmCat->$catName));
					else 
						$jsCatTbl->set('alias_'.$tag, $vmCat->$catAlias);
                    $jsCatTbl->set('description_'.$tag, $vmCat->$catDesc);
                
                    # Automatically fill META-tags if such option is checked
                    if ($this->autofillMeta) {
                        $jsCatTbl->set('meta_title_'.$tag, strip_tags($vmCat->$catName));
                        $jsCatTbl->set('meta_description_'.$tag, strip_tags($vmCat->$catDesc));
                        $jsCatTbl->set('meta_keyword_'.$tag, strip_tags($vmCat->$catName));
                    } else {
                        $jsCatTbl->set('meta_title_'.$tag, strip_tags($vmCat->$customtitle));
                        $jsCatTbl->set('meta_description_'.$tag, strip_tags($vmCat->$metadesc));
                        $jsCatTbl->set('meta_keyword_'.$tag, strip_tags($vmCat->$metakey));
                    }           
                }
            }

			//$jsCatTbl->set('category_parent_id', $vmCat->category_parent_id);
			$jsCatTbl->store();
			
			# Save relations between ids
			$this->vmIdXjsId['category'][$vmCat->virtuemart_category_id] = $jsCatTbl->category_id;
			
			# Save subcat ID, so later I can save real JShop subcat ID to category
			$jsCatIdXvmSubcatId[$jsCatTbl->category_id] = $vmCat->category_parent_id;
		}

		# Now I have all real cats IDs, so I can...
		# Assign subcats to categories
		
		$query = "SELECT category_id FROM `#__jshopping_categories`";

		$this->DB->setQuery($query);
		$this->DB->query();
		
		$jsCatIds = $this->DB->loadResultArray();
        
        if (!empty($jsCatIds[0]))
            foreach ($jsCatIds as $jsCatId) {
                if ($jsCatIdXvmSubcatId[$jsCatId]) {
                    # Category have parent, we can resume
                    $jsCatTbl = JTable::getInstance('Category', 'jshop');
                    
                    $jsCatTbl->load($jsCatId);
                    $jsCatTbl->set('category_parent_id', $this->vmIdXjsId['category'][$jsCatIdXvmSubcatId[$jsCatId]]);

                    $jsCatTbl->store();
                }
            }
	}
	
	# Characteristics
	private function _importCharacteristics() { 
		# Get last ordering
		$ordCnt = $this->_getLastOrdering('characteristics');
		
		$query = "SELECT vm_cust.`virtuemart_custom_id`, vm_cust.`custom_title`
                  FROM `#__virtuemart_customs` AS vm_cust 
                  WHERE vm_cust.`is_cart_attribute` = '0'
                  AND vm_cust.`field_type` <> 'E' AND vm_cust.`field_type` <> 'X' AND vm_cust.`field_type` <> 'P' AND vm_cust.`field_type` <> 'R' AND vm_cust.`field_type` <> 'Z' AND vm_cust.`field_type` <> 'D' AND vm_cust.`field_type` <> 'A' AND vm_cust.`field_type` <> 'Y'";

		$this->DB->setQuery($query);

		foreach ($this->DB->loadObjectList() as $vmCharacteristic) {
			# Create characteristic record
			$jsCharacteristicTbl = JTable::getInstance('ProductField', 'jshop');

			$jsCharacteristicTbl->set('allcats', 1); // Default
			$jsCharacteristicTbl->set('cats', 'a:0:{}'); // JShop makes so
			$jsCharacteristicTbl->set('ordering', ++$ordCnt);
            $jsCharacteristicTbl->set('type', 1); //Default
			
			$jsCharacteristicTbl->set($this->jsLang->get('name'), $vmCharacteristic->custom_title);
			
			$jsCharacteristicTbl->store();
			
            $this->vmIdXjsId['characteristics'][$vmCharacteristic->virtuemart_custom_id] = $jsCharacteristicTbl->id;
            
            $query = "ALTER TABLE `#__jshopping_products` ADD COLUMN `extra_field_". $this->vmIdXjsId['characteristics'][$vmCharacteristic->virtuemart_custom_id] ."` VARCHAR(255) NOT NULL DEFAULT ''";
            $this->DB->setQuery($query);
            $this->DB->query();  
        }
        
        $query = "SELECT vm_cust.`virtuemart_custom_id`, vm_prod_cust.`custom_value`, vm_prod_cust.`virtuemart_product_id`
                  FROM `#__virtuemart_product_customfields` AS vm_prod_cust
                  LEFT JOIN `#__virtuemart_customs` AS vm_cust
                  ON vm_cust.`virtuemart_custom_id` = vm_prod_cust.`virtuemart_custom_id`
                  WHERE vm_cust.`is_cart_attribute` = '0'
                  AND vm_cust.`field_type` <> 'E' AND vm_cust.`field_type` <> 'X' AND vm_cust.`field_type` <> 'P' AND vm_cust.`field_type` <> 'D' AND vm_cust.`field_type` <> 'A' AND vm_cust.`field_type` <> 'Y'";
        
        $this->DB->setQuery($query);

        foreach ($this->DB->loadObjectList() as $vmCharacteristicValue) {
            # Add characteristic field to jshopping_products tbl
            if (!empty($vmCharacteristicValue->custom_value)){
			    $jsProdTbl = JTable::getInstance('Product', 'jshop');
			    
			    $jsProdTbl->load($this->vmIdXjsId['product'][$vmCharacteristicValue->virtuemart_product_id]);
			    
			    $jsProdTbl->set('extra_field_'.$this->vmIdXjsId['characteristics'][$vmCharacteristicValue->virtuemart_custom_id],
							    $vmCharacteristicValue->custom_value);
			    
			    $jsProdTbl->store();
            }
		}
	}

	# Coupons
	private function _importCoupons() {
		$query = "SELECT *
				  FROM `#__virtuemart_coupons`";

		$this->DB->setQuery($query);

		foreach ($this->DB->loadObjectList() as $vmCoupon) {
			# DB import
			$jsCouponTbl = JTable::getInstance('Coupon', 'jshop');

			$jsCouponTbl->set('coupon_type', ($vmCoupon->percent_or_total == 'percent') ? 0 : 1);
			$jsCouponTbl->set('coupon_code', $vmCoupon->coupon_code);
			$jsCouponTbl->set('coupon_value', $vmCoupon->coupon_value);
			$jsCouponTbl->set('tax_id', 0); // None
			$jsCouponTbl->set('used', 0); // Default
			$jsCouponTbl->set('for_user_id', 0); // None
			$jsCouponTbl->set('finished_after_used', ($vmCoupon->coupon_type == 'permanent') ? 0 : 1);
			$jsCouponTbl->set('coupon_publish', 1); // Default
            $jsCouponTbl->set('coupon_start_date', $vmCoupon->coupon_start_date);
			$jsCouponTbl->set('coupon_expire_date', $vmCoupon->coupon_expiry_date);
            
			$jsCouponTbl->store();
		}
	}
	
	# Free attributes
	private function _importFreeAttributes() {
		# Get last ordering
		$ordCnt = $this->_getLastOrdering('free_attributes');

		$query = "SELECT vm_cust.`custom_jplugin_id`, vm_cust.`virtuemart_custom_id`, vm_cust.`custom_title`
                  FROM `#__virtuemart_customs` AS vm_cust 
                  LEFT JOIN `#__extensions` AS ext
                  ON ext.`extension_id` = vm_cust.`custom_jplugin_id` 
                  WHERE ext.`element` = 'textinput'
                  AND ext.`folder` = 'vmcustom' AND ext.`enabled` = '1' AND ext.`access` = '1'
                  AND vm_cust.`field_type` = 'E' AND vm_cust.`custom_jplugin_id` <> 0";

        $this->DB->setQuery($query);;
        
		foreach ($this->DB->loadObjectList() as $vmProd) {
			# Create new free attr DB entry
			$jsFreeAttrTbl = JTable::getInstance('FreeAttribut', 'jshop');
			$jsFreeAttrTbl->set('ordering', ++$ordCnt);
			$jsFreeAttrTbl->set('required', $this->makeFreeAttributesRequired);
			$jsFreeAttrTbl->set($this->jsLang->get('name'), $vmProd->custom_title);
			$jsFreeAttrTbl->store();
            $this->vmIdXjsId['freeAttr'][$vmProd->virtuemart_custom_id] = $jsFreeAttrTbl->id;
   		} 
        
        $query = "SELECT vm_cust.`custom_jplugin_id`, vm_cust.`virtuemart_custom_id`, vm_prod_cust.`custom_value`, vm_prod_cust.`virtuemart_product_id`
                  FROM `#__virtuemart_product_customfields` AS vm_prod_cust
                  LEFT JOIN `#__virtuemart_customs` AS vm_cust
                  ON vm_cust.`virtuemart_custom_id` = vm_prod_cust.`virtuemart_custom_id`
                  LEFT JOIN `#__extensions` AS ext
                  ON ext.`extension_id` = vm_cust.`custom_jplugin_id` 
                  WHERE ext.`element` = 'textinput'
                  AND ext.`folder` = 'vmcustom' AND ext.`enabled` = '1' AND ext.`access` = '1'
                  AND vm_cust.`field_type` = 'E' AND vm_cust.`custom_jplugin_id` <> 0";
        
        $this->DB->setQuery($query);

        foreach ($this->DB->loadObjectList() as $vmProd) {
            $jsProdId = $this->vmIdXjsId['product'][$vmProd->virtuemart_product_id];
            $attr_id = $this->vmIdXjsId['freeAttr'][$vmProd->virtuemart_custom_id];
            $this->DB->setQuery("SELECT COUNT(id) 
                     FROM `#__jshopping_products_free_attr` 
                     WHERE `product_id` = $jsProdId AND `attr_id` = $attr_id");
            if (!$this->DB->loadResult()){         
                $query = "INSERT INTO `#__jshopping_products_free_attr`
                          (product_id, attr_id)
                          VALUES ($jsProdId, $attr_id)";
                $this->DB->setQuery($query);
                $this->DB->query();
            }
        }
        
	}

	# Manufacturers
	private function _importManufacturers() {
		# Get last ordering
		$ordCnt = $this->_getLastOrdering('manufacturers');

        $adv_join = ""; $adv_query = "";   
        foreach ($this->vmConfig['active_languages'] as $tag){
            $vmTag = strtolower(str_replace('-', '_', $tag)); 
            $adv_join .= ' LEFT JOIN `#__virtuemart_manufacturers_'. $vmTag .'` AS vm_mf_desc_'. $vmTag .'
                  ON vm_mf.`virtuemart_manufacturer_id` = vm_mf_desc_'. $vmTag .'.`virtuemart_manufacturer_id`'; 
                  
            $adv_query .= ' vm_mf_desc_'. $vmTag .'.`mf_name` AS mf_name_'. $vmTag .', 
                            vm_mf_desc_'. $vmTag .'.`mf_desc` AS mf_desc_'. $vmTag .', 
                            vm_mf_desc_'. $vmTag .'.`slug` AS mf_alias_'. $vmTag .', 
                            vm_mf_desc_'. $vmTag .'.`mf_url` AS mf_url_'. $vmTag .', '; 
        }

		$query = "SELECT $adv_query vm_mf.`virtuemart_manufacturer_id`, vm_mf.`published`, vm_medias.`file_title`, vm_medias.`file_url`, vm_medias.`file_url_thumb` FROM `#__virtuemart_manufacturers` as vm_mf
                  $adv_join
                  LEFT JOIN `#__virtuemart_manufacturer_medias` AS vm_mf_media
                  ON vm_mf.`virtuemart_manufacturer_id` = vm_mf_media.`virtuemart_manufacturer_id`
                  LEFT JOIN `#__virtuemart_medias` AS vm_medias
                  ON vm_medias.`virtuemart_media_id` = vm_mf_media.`virtuemart_media_id`
                  AND vm_medias.`file_type` = 'manufacturer' AND vm_medias.`file_mimetype` LIKE 'image/%'";

		$this->DB->setQuery($query);
		$this->DB->query();

		if (count($this->DB->loadObjectList()))
		foreach ($this->DB->loadObjectList() as $vmManufacturer) {
			# DB import
			$jsManufacturerTbl = JTable::getInstance('Manufacturer', 'jshop');
            if (!empty($vmManufacturer->file_title)) {
                $createdFilename = $this->_processManufacturerImage($vmManufacturer->file_title, $vmManufacturer->file_url, $vmManufacturer->file_url_thumb);
                        
                $jsManufacturerTbl->set('manufacturer_logo', $createdFilename);
            }

			$jsManufacturerTbl->set('manufacturer_publish', $vmManufacturer->published);
			$jsManufacturerTbl->set('products_page', $this->jsConf->count_products_to_page); // Config
			$jsManufacturerTbl->set('products_row', $this->jsConf->count_products_to_row); // Config
			$jsManufacturerTbl->set('ordering', ++$ordCnt);
            
            $vmMfUrl = '';
            foreach ($this->vmConfig['active_languages'] as $tag){
                if (in_array($tag, $this->jsTags)){
                    $vmTag = strtolower(str_replace('-', '_', $tag)); 
                    $mfName = 'mf_name_'.$vmTag;
                    $mfDesc = 'mf_desc_'.$vmTag;
                    $mfUrl = 'mf_url_'.$vmTag;
                    $mfAlias = 'mf_alias_'.$vmTag;
                    
                    if (!empty($vmManufacturer->$mfUrl)) $vmMfUrl = $vmManufacturer->$mfUrl;
                    $jsManufacturerTbl->set('name_'.$tag, $vmManufacturer->$mfName);
                    
                    # Generate alias if such option is checked
                    if ($this->generateAliases || !$this->deleteJSData)
                        $jsManufacturerTbl->set('alias_'.$tag, $this->_generateAlias('alias_'.$tag, 'manufacturer', $vmManufacturer->$mfName));
                    else 
                    	$jsManufacturerTbl->set('alias_'.$tag, $vmManufacturer->$mfAlias);
                    	        
                    $jsManufacturerTbl->set('description_'.$tag, $vmManufacturer->$mfDesc);
                    
                    # Automatically fill META-tags if such option is checked
                    if ($this->autofillMeta) {
                        $jsManufacturerTbl->set('meta_title_'.$tag, strip_tags($vmManufacturer->$mfName));
                        $jsManufacturerTbl->set('meta_description_'.$tag, strip_tags($vmManufacturer->$mfDesc));
                        $jsManufacturerTbl->set('meta_keyword_'.$tag, strip_tags($vmManufacturer->$mfName));
                    }
                }
            }

            $jsManufacturerTbl->set('manufacturer_url', $vmMfUrl);
            
            $jsManufacturerTbl->store();
			
			# Save relations between ids
			$this->vmIdXjsId['manufacturer'][$vmManufacturer->virtuemart_manufacturer_id] = $jsManufacturerTbl->manufacturer_id;
		}
	}
	
	# Products
	private function _importProducts(){
		$subQuery1 = "SELECT product_price
					  FROM `#__virtuemart_product_prices`
					  WHERE virtuemart_product_id = vm_prod.virtuemart_product_id LIMIT 1";
		$subQuery2 = "SELECT product_currency
					  FROM `#__virtuemart_product_prices`
					  WHERE virtuemart_product_id = vm_prod.virtuemart_product_id LIMIT 1";
		$subQuery3 = "SELECT COUNT(virtuemart_product_price_id)
					  FROM `#__virtuemart_product_prices`
					  WHERE virtuemart_product_id = vm_prod.virtuemart_product_id";
		$subQuery4 = "SELECT rating
					  FROM `#__virtuemart_ratings`
					  WHERE virtuemart_product_id = vm_prod.virtuemart_product_id LIMIT 1";
		$subQuery5 = "SELECT COUNT(virtuemart_rating_review_id)
					  FROM `#__virtuemart_rating_reviews`
					  WHERE virtuemart_product_id = vm_prod.virtuemart_product_id";    
        $subQuery6 = "SELECT product_tax_id
                      FROM `#__virtuemart_product_prices`
                      WHERE virtuemart_product_id = vm_prod.virtuemart_product_id LIMIT 1";
        $subQuery7 = "SELECT product_discount_id
                      FROM `#__virtuemart_product_prices`
                      WHERE virtuemart_product_id = vm_prod.virtuemart_product_id LIMIT 1";
        $subQuery8 = "SELECT calc_value_mathop 
                      FROM `#__virtuemart_calcs` 
                      WHERE virtuemart_calc_id = discount_id LIMIT 1";
        $subQuery9 = "SELECT calc_value 
                      FROM `#__virtuemart_calcs` 
                      WHERE virtuemart_calc_id = discount_id LIMIT 1";
        $subQuery10 = "SELECT publish_up 
                      FROM `#__virtuemart_calcs` 
                      WHERE virtuemart_calc_id = discount_id LIMIT 1";
        $subQuery11 = "SELECT publish_down 
                      FROM `#__virtuemart_calcs` 
                      WHERE virtuemart_calc_id = discount_id LIMIT 1";
        $subQuery12 = "SELECT calc_currency 
                      FROM `#__virtuemart_calcs` 
                      WHERE virtuemart_calc_id = discount_id LIMIT 1";
        $subQuery13 = "SELECT currency_code_3 
                      FROM `#__virtuemart_currencies` 
                      WHERE virtuemart_currency_id = product_currency LIMIT 1";
        $subQuery14 = "SELECT currency_code_3 
                      FROM `#__virtuemart_currencies` 
                      WHERE virtuemart_currency_id = discount_currency LIMIT 1";    
        $subQuery15 = "SELECT GROUP_CONCAT(vm_prod_medias.`virtuemart_media_id` ORDER BY vm_prod_medias.`ordering` ASC)
                      FROM `#__virtuemart_product_medias` AS vm_prod_medias
                      WHERE vm_prod_medias.`virtuemart_product_id` = vm_prod.`virtuemart_product_id`";
        $subQuery16 = "SELECT GROUP_CONCAT(vm_prod_cat.`virtuemart_category_id` ORDER BY vm_prod_cat.`ordering` ASC)
                      FROM `#__virtuemart_product_categories` AS vm_prod_cat
                      WHERE vm_prod_cat.`virtuemart_product_id` = vm_prod.`virtuemart_product_id`";  
		                                                                      
        $adv_join = ""; $adv_query = "";   
        foreach ($this->vmConfig['active_languages'] as $tag){
            $vmTag = strtolower(str_replace('-', '_', $tag)); 
            $adv_join .= ' LEFT JOIN `#__virtuemart_products_'. $vmTag .'` AS vm_prod_desc_'. $vmTag .'
                  ON vm_prod.`virtuemart_product_id` = vm_prod_desc_'. $vmTag .'.`virtuemart_product_id`'; 
                  
            $adv_query .= ' vm_prod_desc_'. $vmTag .'.`metadesc` AS metadesc_'. $vmTag .', 
                            vm_prod_desc_'. $vmTag .'.`metakey` AS metakey_'. $vmTag .', 
                            vm_prod_desc_'. $vmTag .'.`customtitle` AS customtitle_'. $vmTag .', 
                            vm_prod_desc_'. $vmTag .'.`product_s_desc` AS prod_s_desc_'. $vmTag .', 
                            vm_prod_desc_'. $vmTag .'.`product_desc` AS prod_desc_'. $vmTag .',
                            vm_prod_desc_'. $vmTag .'.`product_name` AS prod_name_'. $vmTag .', 
                            vm_prod_desc_'. $vmTag .'.`slug` AS prod_slug_'. $vmTag .', '; 
        }
		$query = "SELECT vm_prod.*, $adv_query 
                         ($subQuery15) AS virtuemart_media_ids,
                         ($subQuery16) AS virtuemart_category_ids, 
						 ($subQuery1) AS product_price,
						 ($subQuery2) AS product_currency,
						 ($subQuery3) AS product_add_price_quantity,
						 ($subQuery4) AS product_rating,
						 ($subQuery5) AS product_reviews_quantity,
						 ($subQuery6) AS product_tax_id,
                         ($subQuery7) AS discount_id,  
                         ($subQuery8) AS calc_value_mathop,
                         ($subQuery9) AS discount_price,
                         ($subQuery10) AS publish_up,
                         ($subQuery11) AS publish_down,
                         ($subQuery12) AS discount_currency,
                         ($subQuery13) AS product_currency_code,
                         ($subQuery14) AS discount_currency_code,
						 vm_prod_manufacturer.virtuemart_manufacturer_id

				  FROM `#__virtuemart_products` AS vm_prod  
				  LEFT JOIN `#__virtuemart_shoppergroups` AS vm_shopper_group USING (virtuemart_vendor_id)

                  LEFT JOIN `#__virtuemart_product_manufacturers` AS vm_prod_manufacturer USING (virtuemart_product_id)
				  $adv_join
                  WHERE vm_shopper_group.`default` = 1";
                  
		$this->DB->setQuery($query);
		$this->DB->query();

		$total = count($this->DB->loadObjectList());
		$cnt = 0;
		if (count($this->DB->loadObjectList()) > 0)
		foreach ($this->DB->loadObjectList() as $vmProd) {
			# Print start message	 
			echo "&nbsp;&nbsp;&nbsp;<b>(".($cnt + 1)."/$total)</b>... ";
			
			$cnt++;
            flush();
			@ob_flush();
           
			# DB import
			$jsProdTbl = JTable::getInstance('Product', 'jshop');

			$jsProdTbl->set('product_ean', $vmProd->product_sku);   
			$jsProdTbl->set('product_quantity', $vmProd->product_in_stock);  
			
			# Is available date in the past?
			$jsProdTbl->set('product_availability', (time() < $vmProd->product_available_date) ? 1 : 0);   

			$jsProdTbl->set('product_date_added', $vmProd->created_on);  
			$jsProdTbl->set('date_modify', $vmProd->modified_on);  
			$jsProdTbl->set('product_publish', $vmProd->published);  
            
            if (!$vmProd->product_tax_id){
                reset($this->vmIdXjsId['tax']); 
                $jsProdTbl->set('product_tax_id', $this->vmIdXjsId['tax'][key($this->vmIdXjsId['tax'])]);    
            }
            else $jsProdTbl->set('product_tax_id', $this->vmIdXjsId['tax'][$vmProd->product_tax_id]);
			$jsProdTbl->set('product_template', 'default'); // Default
			$jsProdTbl->set('product_url', $vmProd->product_url);
			$jsProdTbl->set('product_old_price', '0.00'); // Default
			$jsProdTbl->set('product_buy_price', '0.00'); // Default       
			
			# Price
			# JShop does not support discounts for products, so I should recalculate price if discount present
			$discountActual = true;

			if (!empty($vmProd->publish_up) && date('Y-m-d H:i:s') < $vmProd->publish_up)
				$discountActual = false;
			
			if (!empty($vmProd->publish_down) && date('Y-m-d H:i:s') > $vmProd->publish_down)
				$discountActual = false;
			
            $jsProdTbl->set('product_price', $this->_convertPrice($vmProd->calc_value_mathop, $vmProd->product_price, $vmProd->discount_price, $vmProd->product_currency_code, $vmProd->discount_currency_code, $discountActual));

			$jsProdTbl->set('product_weight', $vmProd->product_weight);     

            $categoryArray = explode(',', $vmProd->virtuemart_category_ids);
            if (count($categoryArray) > 0){
                foreach ($categoryArray as $value){
                	if (!empty($value)){
                    	$data = new stdClass();
                    	$data->category_id = $this->vmIdXjsId['category'][$value];
                    	$data->product_id = $vmProd->virtuemart_product_id;
                    	$this->DB->insertObject('#__jshopping_products_to_categories', $data, id); 
                	}
                }
            }
			# VMart can have many manufacturers for one product, I set first meeted
			$jsProdTbl->set('product_manufacturer_id', $this->vmIdXjsId['manufacturer'][$vmProd->virtuemart_manufacturer_id]);

			$jsProdTbl->set('product_is_add_price', ($vmProd->product_add_price_quantity > 1) ? 1 : 0);

			# Transpose VMart max mark system (5) to JShop

			$jsProdTbl->set('average_rating', ($vmProd->product_rating * $this->jsConf->max_mark) / 5);

			$jsProdTbl->set('reviews_count', $vmProd->product_reviews_quantity);
			$jsProdTbl->set('vendor_id', $this->vmIdXjsId['vendor'][$vmProd->virtuemart_vendor_id]);  
			
			foreach ($this->vmConfig['active_languages'] as $tag){
                if (in_array($tag, $this->jsTags)){
                    $vmTag = strtolower(str_replace('-', '_', $tag)); 
                    $prodSDesc = 'prod_s_desc_'.$vmTag;
                    $prodDesc = 'prod_desc_'.$vmTag;
                    $prodName = 'prod_name_'.$vmTag;
                    $prodSlug = 'prod_slug_'.$vmTag;

                    $metadesc = 'metadesc_'.$vmTag;
                    $metakey = 'metakey_'.$vmTag;
                	$customtitle = 'customtitle_'.$vmTag;
                    
                    # Product name
                    $prodNameAttr = $vmProd->$prodName;
                    
                    # Add subproduct attributes to product title if such option is enabled
                    if (!empty($vmProd->product_parent_id) && $this->addAttrsToSubproductTitle)
                        $prodNameAttr .= ' '.$this->_getSubproductAttrsStr($vmProd->virtuemart_product_id);
                        
                    $jsProdTbl->set('name_'.$tag, $prodNameAttr);      

                    # Generate alias if such option is checked
                    if ($this->generateAliases || !$this->deleteJSData)
                        $jsProdTbl->set('alias_'.$tag, $this->_generateAlias('alias_'.$tag, 'product', $prodNameAttr));   
                    
                    $jsProdTbl->set('short_description_'.$tag, $vmProd->$prodSDesc);     
                    $jsProdTbl->set('description_'.$tag, $vmProd->$prodDesc);         
                    
                    # Automatically fill META-tags if such option is checked
                    if ($this->autofillMeta) {
                        $jsProdTbl->set('meta_title_'.$tag, strip_tags($vmProd->$prodName));           
                        $jsProdTbl->set('meta_description_'.$tag, strip_tags($vmProd->$prodName)); 
                        $jsProdTbl->set('meta_keyword_'.$tag, strip_tags($vmProd->$prodName));    
                    } else {
                        $jsProdTbl->set('meta_title_'.$tag, strip_tags($vmProd->$customtitle));
                        $jsProdTbl->set('meta_description_'.$tag, strip_tags($vmProd->$metadesc));
                        $jsProdTbl->set('meta_keyword_'.$tag, strip_tags($vmProd->$metakey));
                    }
                }
            }    
           
			$jsProdTbl->store();
			
			# Save relations between ids
			$this->vmIdXjsId['product'][$vmProd->virtuemart_product_id] = $jsProdTbl->product_id;   
			
			# Save product currency for later use
			$this->vmProductsCurrencies[$vmProd->virtuemart_product_id] = $vmProd->product_currency;    
		
			if (!empty($vmProd->virtuemart_media_ids)) {
				# Product images
            	$query = "SELECT vm_medias.`file_title`, vm_medias.`file_url_thumb`, vm_medias.`file_url` 
                     	  FROM `#__virtuemart_medias` AS vm_medias 
                     	  WHERE vm_medias.`virtuemart_media_id` IN (". $vmProd->virtuemart_media_ids .") AND vm_medias.`file_type` = 'product' AND vm_medias.`file_mimetype` LIKE 'image/%'
                     	  ORDER BY FIND_IN_SET(vm_medias.`virtuemart_media_id`, '". $vmProd->virtuemart_media_ids ."')";
	            $this->DB->setQuery($query);
			
            	if ($this->DB->loadObjectList())
            	foreach($this->DB->loadObjectList() as $value){
                	if (!empty($value->file_title) && is_file(JPATH_SITE.'/'.$value->file_url_thumb) && file_exists(JPATH_SITE.'/'.$value->file_url)) {    
                    	# Process files
                    	$imgsInfo = new stdClass();
                    	$imgsInfo->srcFullImgPath = JPATH_SITE.'/'.$value->file_url;
                    	$imgsInfo->ImgBasename = $value->file_title;
                    	$imgsInfo->srcThumbImgPath = JPATH_SITE.'/'.$value->file_url_thumb;
                    
                    	$this->_processProductImages($imgsInfo, $jsProdTbl->product_id);
                	} else if (!empty($value->file_title) && file_exists(JPATH_SITE.'/'.$value->file_url)){
                		$imgsInfo = new stdClass();
                    	$imgsInfo->srcFullImgPath = JPATH_SITE.'/'.$value->file_url;
                    	$imgsInfo->ImgBasename = $value->file_title;
                    	$this->_processProductImages($imgsInfo, $jsProdTbl->product_id);	
                	}  
            	}
			}
			# Print finished message
			$this->_printImportingFooter();
		}
	}
	
	# Products relations
	private function _importProductsRelations() {
		//$query = "SELECT *  FROM `#__virtuemart_product_relations`"; not working!!!!
            //--------------------------------------------------//
            $query = "SELECT C.`virtuemart_product_id` , C.`custom_value` FROM `ueh5b_virtuemart_product_customfields` AS C
                        LEFT JOIN `ueh5b_virtuemart_customs` AS field ON C.`virtuemart_custom_id` = field.`virtuemart_custom_id`
                        WHERE field.`field_type` = 'R'";
		$this->DB->setQuery($query);
        $data = new stdClass();
		foreach ($this->DB->loadObjectList() as $vmProdRelation) {
			$data->product_id = $this->vmIdXjsId['product'][$vmProdRelation->virtuemart_product_id];
                        $data->product_related_id =  $this->vmIdXjsId['product'][$vmProdRelation->custom_value];
			$this->DB->insertObject('#__jshopping_products_relations', $data, id);
                        
                        
//                        $productRelatedIds = explode('|', $vmProdRelation->custom_value);
//
//			foreach ($productRelatedIds as $relProdId) {
//				$data->product_related_id = $this->vmIdXjsId['product'][$relProdId];
//				$this->DB->insertObject('#__jshopping_products_relations', $data, id);
//			}
		}

		# Make subproducts related if such option is checked
		if ($this->makeSubproductsRelative) {
                        $prodsToCats = new stdClass(); 
			$query = "SELECT virtuemart_product_id, product_parent_id
					  FROM `#__virtuemart_products`
					  WHERE product_parent_id <> 0";

			$this->DB->setQuery($query); 
			foreach ($this->DB->loadObjectList() as $vmProd) {
                $data->product_related_id = $this->vmIdXjsId['product'][$vmProd->virtuemart_product_id];
                $data->product_id = $this->vmIdXjsId['product'][$vmProd->product_parent_id];
                $this->DB->insertObject('#__jshopping_products_relations', $data, id);

                
                //------------- add category-------------------//               
                $query_cats = "SELECT virtuemart_category_id FROM `#__virtuemart_product_categories`
					  WHERE virtuemart_product_id = $vmProd->product_parent_id";
                $this->DB->setQuery($query_cats);
                foreach ($this->DB->loadObjectList() as $vmCats) {
                  $prodsToCats->product_id = $this->vmIdXjsId['product'][$vmProd->virtuemart_product_id];
                  $prodsToCats->category_id = $this->vmIdXjsId['category'][$vmCats->virtuemart_category_id];
                  $this->DB->insertObject('#__jshopping_products_to_categories', $prodsToCats);
                }
                //------------- end adding category-------------------//
                
                
			}
		}
	}

	# Products reviews
	private function _importProductsReviews() {
		$query = "SELECT prod_reviews.*, users.username, users.email
				  FROM `#__virtuemart_rating_reviews` AS prod_reviews
				  LEFT JOIN `#__users` AS users
				  ON prod_reviews.modified_by = users.id";

		$this->DB->setQuery($query);

		foreach ($this->DB->loadObjectList() as $vmProdReview) {
			$jsProductReviewTbl = JTable::getInstance('Review', 'jshop');
			
			$jsProductReviewTbl->set('product_id', $this->vmIdXjsId['product'][$vmProdReview->virtuemart_product_id]);
			$jsProductReviewTbl->set('user_id', $vmProdReview->modified_by);
			$jsProductReviewTbl->set('user_name', $vmProdReview->username);
			$jsProductReviewTbl->set('user_email', $vmProdReview->email);
			$jsProductReviewTbl->set('time', $vmProdReview->modified_on);
			$jsProductReviewTbl->set('review', $vmProdReview->comment);
			$jsProductReviewTbl->set('ip', $vmProdReview->lastip);
			
			# Transpose curr rating to JShop max rating
			$jsProductReviewTbl->set('mark', ($vmProdReview->review_rating * $this->jsConf->max_mark) / 5);
			
			$jsProductReviewTbl->set('publish', $vmProdReview->published);
			
			$jsProductReviewTbl->store();
		}
	}
	
	# Products shipment prices
	private function _importProductsShipmentPrices() {
		$defaultShopperSubQuery = "SELECT virtuemart_shoppergroup_id
								   FROM `#__virtuemart_shoppergroups`
								   WHERE virtuemart_vendor_id = 0 AND `default` = 1";
								   
		$query = "SELECT prod_prices_quantity.*,
						 prod_prices_base.product_price AS base_price,
						 prod_prices_base.product_currency AS base_currency
						 
				  FROM `#__virtuemart_product_prices` AS prod_prices_quantity
				  
				  LEFT JOIN `#__virtuemart_product_prices` AS prod_prices_base ON prod_prices_quantity.virtuemart_product_id = prod_prices_base.virtuemart_product_id
				  LEFT JOIN `#__virtuemart_products` AS prod ON prod_prices_quantity.virtuemart_product_id = prod.virtuemart_product_id
				  LEFT JOIN `#__virtuemart_shoppergroups` AS vm_shopper_group USING (virtuemart_vendor_id)

				  WHERE (prod_prices_quantity.price_quantity_start <> 0 OR prod_prices_quantity.price_quantity_end) <> 0 AND
						(prod_prices_base.price_quantity_start = 0 AND prod_prices_base.price_quantity_end = 0) AND
						prod_prices_quantity.virtuemart_shoppergroup_id = COALESCE(vm_shopper_group.virtuemart_shoppergroup_id, ($defaultShopperSubQuery))";

		$this->DB->setQuery($query);
		$this->DB->query();
        
		foreach ($this->DB->loadObjectList() as $vmProdShipmentPrice) {
			# Convert prices considering currency
			$basePrice = $this->_convertDiscountPrice($vmProdShipmentPrice->base_price,
											  $vmProdShipmentPrice->base_currency);
			
			$discountPrice = $this->_convertDiscountPrice($vmProdShipmentPrice->product_price,
												  $vmProdShipmentPrice->product_currency);
													  
			# DB import
			$jsProdShipmentPriceTbl = JTable::getInstance('ProductPrice', 'jshop');

			$jsProdShipmentPriceTbl->set('product_id',
										 $this->vmIdXjsId['product'][$vmProdShipmentPrice->virtuemart_product_id]);

			# 1 - price, 2 - percent
			switch ($this->jsConf->product_price_qty_discount) {
				case 1:
					$jsProdShipmentPriceTbl->set('discount', $basePrice - $discountPrice);
					
					break;
				case 2:
					$jsProdShipmentPriceTbl->set('discount',
												 abs(($discountPrice - $basePrice) / $basePrice) * 100);
					break;
			}

			$jsProdShipmentPriceTbl->set('product_quantity_start',
									 $vmProdShipmentPrice->price_quantity_start);
			$jsProdShipmentPriceTbl->set('product_quantity_finish',
									 $vmProdShipmentPrice->price_quantity_end);
			
			$jsProdShipmentPriceTbl->store();
		}
	}
    
	# Shipping methods
	private function _importShippingMethods() {
		# Get last ordering
		$ordCnt = $this->_getLastOrdering('shipping_methods');
        $adv_query = ''; $adv_join = '';
		foreach ($this->vmConfig['active_languages'] as $tag){
            $vmTag = strtolower(str_replace('-', '_', $tag)); 
            $adv_join .= ' LEFT JOIN `#__virtuemart_shipmentmethods_'. $vmTag .'` AS vm_sm_desc_'. $vmTag .'
                  ON vm_sm.`virtuemart_shipmentmethod_id` = vm_sm_desc_'. $vmTag .'.`virtuemart_shipmentmethod_id`'; 
                  
            $adv_query .= ', vm_sm_desc_'. $vmTag .'.`shipment_name` AS sm_name_'. $vmTag .', 
                            vm_sm_desc_'. $vmTag .'.`shipment_desc` AS sm_desc_'. $vmTag .' '; 
        }
		$query = "SELECT vm_sm.* $adv_query
				  FROM `#__virtuemart_shipmentmethods` AS vm_sm
                  $adv_join
                  WHERE vm_sm.`shipment_params` <> ''
				  ORDER BY vm_sm.`ordering`";

		$this->DB->setQuery($query);

		# DB import
		if (count($this->DB->loadObjectList()))
		foreach ($this->DB->loadObjectList() as $shippingMethod) {
			# DB import of shipping method
			$jsShippingMethodTbl = JTable::getInstance('ShippingMethod', 'jshop');
			foreach ($this->vmConfig['active_languages'] as $tag){
                if (in_array($tag, $this->jsTags)){
                    $vmTag = strtolower(str_replace('-', '_', $tag)); 
                    $smName = 'sm_name_'.$vmTag;
                    $smDesc = 'sm_desc_'.$vmTag; 
                    $jsShippingMethodTbl->set('name_'.$tag, $shippingMethod->$smName);
                    $jsShippingMethodTbl->set('description_'.$tag, $shippingMethod->$smDesc);
                }
			}
			$jsShippingMethodTbl->set('published', 1); // Default
			$jsShippingMethodTbl->set('ordering', ++$ordCnt);
            
			$jsShippingMethodTbl->store();
			# DB import of shipping method price
			$jsShippingMethodPriceTbl = JTable::getInstance('ShippingMethodPrice', 'jshop');
		    $smParams = $this->_parseVmParams($shippingMethod->shipment_params);
			$jsShippingMethodPriceTbl->set('shipping_method_id', $jsShippingMethodTbl->shipping_id);
			if (!$smParams['tax_id']){
                reset($this->vmIdXjsId['tax']); 
                $jsShippingMethodPriceTbl->set('shipping_tax_id', $this->vmIdXjsId['tax'][key($this->vmIdXjsId['tax'])]);    
            }
            else $jsShippingMethodPriceTbl->set('shipping_tax_id', $this->vmIdXjsId['tax'][$smParams['tax_id']]);
			
            $jsShippingMethodPriceTbl->set('shipping_stand_price', $smParams['cost']);
			
			$jsShippingMethodPriceTbl->store();
			
			# DB import of shipping method price countries
			if (count($smParams['countries']))
				foreach ($smParams['countries'] as $countryId) {
					$countryId = $this->_getJSCountryID($countryId);
					if (!empty($countryId)) {
						# JShop have no JTable for ShippingMethodPriceCountries, so I add manually
						$query = "INSERT INTO `#__jshopping_shipping_method_price_countries`
								  (country_id, sh_pr_method_id)
								  VALUES ($countryId, $jsShippingMethodPriceTbl->sh_pr_method_id)";
						$this->DB->setQuery($query);
						$this->DB->query();
					}
				}
			# DB import of shipping method weight ranges and prices for them
			$jsShippingMethodPriceWeightTbl = JTable::getInstance('ShippingMethodPriceWeight', 'jshop');
	
			$jsShippingMethodPriceWeightTbl->set('sh_pr_method_id',
												 $jsShippingMethodPriceTbl->sh_pr_method_id);
			$jsShippingMethodPriceWeightTbl->set('shipping_price', $smParams['cost']);
			$jsShippingMethodPriceWeightTbl->set('shipping_weight_from', $smParams['weight_start']);
			$jsShippingMethodPriceWeightTbl->set('shipping_weight_to', $smParams['weight_stop']);
			$jsShippingMethodPriceWeightTbl->set('shipping_package_price', $smParams['package_fee']);
												 
			$jsShippingMethodPriceWeightTbl->store();
		}
	}

	# Taxes
	private function _importTaxes() {
		$query = "SELECT virtuemart_calc_id, calc_name, calc_value FROM `#__virtuemart_calcs` 
                    WHERE `calc_kind` = 'Tax' AND `calc_value_mathop` = '+%'";

		$this->DB->setQuery($query);
		$this->DB->query();

		foreach ($this->DB->loadObjectList() as $vmTaxRate) {
			# DB import
			$jsTaxTbl = JTable::getInstance('Tax', 'jshop');

			$jsTaxTbl->set('tax_name', $vmTaxRate->calc_name);
			$jsTaxTbl->set('tax_value', $vmTaxRate->calc_value);
			
			$jsTaxTbl->store();
			
			# Save relations between ids
			$this->vmIdXjsId['tax'][$vmTaxRate->virtuemart_calc_id] = $jsTaxTbl->tax_id;
		}
	}

	# User groups
	private function _importUserGroups() {
		$query = "SELECT *
				  FROM `#__virtuemart_shoppergroups`";

		$this->DB->setQuery($query);

		foreach ($this->DB->loadObjectList() as $vmShopperGroup) {
			# DB import
			$jsUserGroupTbl = JTable::getInstance('UserGroup', 'jshop');

			$jsUserGroupTbl->set('usergroup_name', $vmShopperGroup->shopper_group_name);
			$jsUserGroupTbl->set('usergroup_discount', $vmShopperGroup->shopper_group_discount);
			$jsUserGroupTbl->set('usergroup_description', $vmShopperGroup->shopper_group_desc);
			$jsUserGroupTbl->set('usergroup_is_default', 0); // Its not default
			
			$jsUserGroupTbl->store();
			
			# Save relations between ids
			$this->vmIdXjsId['usergroup'][$vmShopperGroup->virtuemart_shoppergroup_id] = $jsUserGroupTbl->usergroup_id;
		}
	}

	# Users extended data
	private function _importUsersExtendedData() {
       $query = "SELECT j_users.`email`, vm_user_inf.*, vm_user_sh_gr.`virtuemart_shoppergroup_id`, j_users.`username`, vm_users.*
				  FROM `#__virtuemart_vmusers` AS vm_users 
				  LEFT JOIN `#__virtuemart_vmuser_shoppergroups` AS vm_user_sh_gr USING(virtuemart_user_id)
                  LEFT JOIN `#__virtuemart_userinfos` AS vm_user_inf
                  ON vm_user_inf.`virtuemart_user_id` = vm_users.`virtuemart_user_id`
                  LEFT JOIN `#__users` AS j_users
                  ON j_users.`id` = vm_users.`virtuemart_user_id`";

		$this->DB->setQuery($query);

		foreach ($this->DB->loadObjectList() as $vmUserInfo) {
			if (isset($vmUserInfo->virtuemart_user_id) && !empty($vmUserInfo->virtuemart_user_id) && $vmUserInfo->virtuemart_user_id){
                # Delete extended data for this user
                $query = "DELETE FROM `#__jshopping_users`
                          WHERE user_id = $vmUserInfo->virtuemart_user_id";

                $this->DB->setQuery($query);
                $this->DB->query();

                # DB import
                $jsUserShopTbl = JTable::getInstance('UserShop', 'jshop');

                $jsUserShopTbl->set('user_id', $vmUserInfo->virtuemart_user_id);

                if (!empty($vmUserInfo->virtuemart_shoppergroup_id))
                    $jsUserShopTbl->set('usergroup_id', $this->vmIdXjsId['usergroup'][$vmUserInfo->virtuemart_shoppergroup_id]);
                else
                    $jsUserShopTbl->set('usergroup_id', 1); // Default

                $jsUserShopTbl->set('f_name', $vmUserInfo->first_name);
                $jsUserShopTbl->set('l_name', $vmUserInfo->last_name);
                $jsUserShopTbl->set('firma_name', $vmUserInfo->company);
                $jsUserShopTbl->set('email', $vmUserInfo->email);
                if (!empty($vmUserInfo->address_2)) $jsUserShopTbl->set('street', $vmUserInfo->address_1.', '.$vmUserInfo->address_2);
                    else $jsUserShopTbl->set('street', $vmUserInfo->address_1);      
                $jsUserShopTbl->set('zip', $vmUserInfo->zip);
                $jsUserShopTbl->set('city', $vmUserInfo->city);
                $jsUserShopTbl->set('state', $this->_getVMStateName($vmUserInfo->virtuemart_state_id));

                $jsUserShopTbl->set('country', $this->_getJSCountryID($vmUserInfo->virtuemart_country_id));

                $jsUserShopTbl->set('phone', $vmUserInfo->phone_1);
                $jsUserShopTbl->set('fax', $vmUserInfo->fax);
                $jsUserShopTbl->set('u_name', $vmUserInfo->username);

                $jsUserShopTbl->set('title', ($vmUserInfo->title == 'Mr') ? 1 : 2);

                # When we set primary_key standart joomla $tbl->store() method will always make UPDATE
                # and rows will never be inserted. So we should use more low-level code to solve this:
                # $this->DB->insertObject($tbl->getTableName(), $tbl, $tbl->getKeyName());
                $this->DB->insertObject($jsUserShopTbl->getTableName(), $jsUserShopTbl,
                                        $jsUserShopTbl->getKeyName());
            }
		}
	}

	# Vendors
	private function _importVendors() {
		# VMart keep store info in vm_vendor table with vendor_id=1, other records are vendors
		if ($this->importLanguage == 'auto') $vmTag = strtolower(str_replace('-', '_', $this->jsLang->lang));
        else $vmTag = strtolower(str_replace('-', '_', $this->importLanguage));     

         $query = 'SELECT `virtuemart_user_id` FROM `#__virtuemart_vmusers` WHERE `virtuemart_vendor_id`=' . (int)$vmVendor->virtuemart_vendor_id;
            $this->DB->setQuery($query);
            $userVendor = $this->DB->loadResult();
        
		$query = "SELECT j_users.`email`, vm_user_inf.*, vm_vendors.`virtuemart_vendor_id`, vm_vendors.`vendor_name`, vm_vendors_desc.*,
                  vm_medias.`file_title`, vm_medias.`file_url`, vm_medias.`file_url_thumb`     
				  FROM `#__virtuemart_vendors` AS vm_vendors
                  LEFT JOIN `#__virtuemart_vendors_$vmTag` AS vm_vendors_desc
                  ON vm_vendors.`virtuemart_vendor_id` = vm_vendors_desc.`virtuemart_vendor_id`
                  LEFT JOIN `#__virtuemart_vendor_medias` AS vm_vendors_media
                  ON vm_vendors.`virtuemart_vendor_id` = vm_vendors_media.`virtuemart_vendor_id`
                  LEFT JOIN `#__virtuemart_medias` AS vm_medias
                  ON vm_medias.`virtuemart_media_id` = vm_vendors_media.`virtuemart_media_id`
                  AND vm_medias.`file_type` = 'vendor' AND vm_medias.`file_mimetype` LIKE 'image/%'
                  LEFT JOIN `#__virtuemart_vmusers` AS vm_users
                  ON vm_users.`virtuemart_vendor_id`= vm_vendors.`virtuemart_vendor_id` AND vm_users.`user_is_vendor` = 1
				  LEFT JOIN `#__virtuemart_userinfos` AS vm_user_inf
                  ON vm_user_inf.`virtuemart_user_id` = vm_users.`virtuemart_user_id`
                  LEFT JOIN `#__users` AS j_users
                  ON j_users.`id` = vm_users.`virtuemart_user_id`
                  ";

		$this->DB->setQuery($query);
		$this->DB->query();
        
		if (count($this->DB->loadObjectList()))
		foreach ($this->DB->loadObjectList() as $vmVendor) {
			# DB import
			$jsVendorTbl = JTable::getInstance('Vendor', 'jshop');

			$jsVendorTbl->set('shop_name', $vmVendor->vendor_store_name);    
			$jsVendorTbl->set('company_name', $vmVendor->vendor_name);       
			$jsVendorTbl->set('url', $vmVendor->vendor_url);                 

			# Processing logo
		
			# In JShop this is URL of picture, in VMart - stored picture.
			# So I will copy it to JShop path and save URL of it in DB

			switch ($this->vendorLogo) {
				case 'full':
					if (!empty($vmVendor->file_title) && file_exists(JPATH_SITE.'/'.$vmVendor->file_url)) {
						# Get first available filename if file with given filename already exists
						$availFilename = MMLib::getAvailableFileName($this->jsConf->image_vendors_path.'/'.$vmVendor->file_title);									 
						# File copy
						copy(JPATH_SITE.'/'.$vmVendor->file_url, $this->jsConf->image_vendors_path.'/'.$availFilename);
						chmod($this->jsConf->image_vendors_path.'/'.$availFilename, 0777);

						$jsVendorTbl->set('logo', $this->jsConf->image_vendors_live_path.'/'.$availFilename);
					}
					
					break;
				case 'thumb':
					if (!empty($vmVendor->file_title) && file_exists(JPATH_SITE.'/'.$vmVendor->file_url_thumb)) {
						# Get first available filename if file with given filename already exists
						$availFilename = MMLib::getAvailableFileName($this->jsConf->image_vendors_path.'/'.$vmVendor->file_title);
						# File copy
						copy(JPATH_SITE.'/'.$vmVendor->file_url_thumb, $this->jsConf->image_vendors_path.'/'.$availFilename);
						chmod($this->jsConf->image_vendors_path.'/'.$availFilename, 0777);

						$jsVendorTbl->set('logo', $this->jsConf->image_vendors_live_path.'/'.$availFilename);
					}elseif (!empty($vmVendor->file_title) && file_exists(JPATH_SITE.'/'.$vmVendor->file_url)) {
                        # Get first available filename if file with given filename already exists
                        $availFilename = MMLib::getAvailableFileName($this->jsConf->image_vendors_path.'/'. $vmVendor->file_title);                                     
                        # File copy
                        copy(JPATH_SITE.'/'.$vmVendor->file_url, $this->jsConf->image_vendors_path.'/'.$availFilename);
                        chmod($this->jsConf->image_vendors_path.'/'.$availFilename, 0777);

                        $jsVendorTbl->set('logo', $this->jsConf->image_vendors_live_path.'/'.$availFilename);
                    }
					
					break;
			}
			
            $query = "SELECT COUNT(main) FROM `#__jshopping_vendors` WHERE main = '1'";
            $this->DB->setQuery($query);
            $main = $this->DB->loadResult();
            if (!$main) $jsVendorTbl->set('main', 1);
                else $jsVendorTbl->set('main', 0);      
            $jsVendorTbl->set('additional_information', $vmVendor->vendor_store_desc);

			$jsVendorTbl->set('user_id', 0); // None     
 
            if (!empty($vmVendor->address_2)) $jsVendorTbl->set('adress', $vmVendor->address_1.', '.$vmVendor->address_2);
                else $jsVendorTbl->set('adress', $vmVendor->address_1); 
			$jsVendorTbl->set('city', $vmVendor->city);
			$jsVendorTbl->set('zip', $vmVendor->zip);    
			
			# Get state name
			$jsVendorTbl->set('state', $this->_getVMStateName($vmVendor->virtuemart_state_id)); 
			
			# Get country ID
			$jsVendorTbl->set('country', $this->_getJSCountryID($vmVendor->virtuemart_country_id));
			
			$jsVendorTbl->set('f_name', $vmVendor->first_name);
			$jsVendorTbl->set('l_name', $vmVendor->last_name);
            $jsVendorTbl->set('middlename', $vmVendor->middle_name);  
			$jsVendorTbl->set('phone', $vmVendor->phone_1);
			$jsVendorTbl->set('fax', $vmVendor->fax);
			$jsVendorTbl->set('email', $vmVendor->email);

			$jsVendorTbl->store();
			
			# Save relations between ids
			$this->vmIdXjsId['vendor'][$vmVendor->virtuemart_vendor_id] = $jsVendorTbl->id;
		}
	}
	
	# Import
	private function _import() {
		# Order of the import:
		# 1. Categories, manufacturers, taxes and vendors
		# 2. Products
		# 3. All other

		# Categories
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_CATEGORIES);
		$this->_importCategories();
		$this->_printImportingFooter();

		# Manufacturers
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_MANUFACTURERS);
		$this->_importManufacturers();
		$this->_printImportingFooter();
		
		# Taxes
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_TAXES);
		$this->_importTaxes();
		$this->_printImportingFooter();
		
		# Vendors
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_VENDORS);
		$this->_importVendors();
		$this->_printImportingFooter();
		
		# Products
		echo '<br />';
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_PRODUCTS);
		echo '<br />';
		$this->_importProducts();
		$this->_printImportingFooter();
		echo '<br />';
	
		# Products relations
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_PRODUCTSRELATIONS);
		$this->_importProductsRelations();
		$this->_printImportingFooter();
        
		# Product reviews
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_PRODUCTSREVIEWS);
		$this->_importProductsReviews();
		$this->_printImportingFooter();
		
		# Product shipment prices
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_PRODUCTSSHIPMENTPRICES);
		$this->_importProductsShipmentPrices();
		$this->_printImportingFooter();
		
		# Attributes
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_ATTRIBUTES);
		$this->_importAttributes();
        $this->_importProductAttributes();
		$this->_printImportingFooter();
		
		# Characteristics
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_CHARACTERISTICS);
		$this->_importCharacteristics();
		$this->_printImportingFooter();
		
		# Coupons
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_COUPONS);
		$this->_importCoupons();
		$this->_printImportingFooter();
		
		# Free attributes
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_FREEATTRIBUTES);
		$this->_importFreeAttributes();
		$this->_printImportingFooter();
		
		# Shipping methods
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_SHIPPINGMETHODS);
		$this->_importShippingMethods();
		$this->_printImportingFooter();
			
		# User groups
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_USERGROUPS);
		$this->_importUserGroups();
		$this->_printImportingFooter();

		# Users extended data
		$this->_printImportingHeader(_JSHOP_IE_VMIMPORT_USERSEXTENDEDDATA);
		$this->_importUsersExtendedData();
		$this->_printImportingFooter();   
	}

	# Start view
	public function view() {
		# Don't work in constructor
		$this->loadLanguageFile();
		
		# Checking for JShop is installed
		#(actually not needed, if this script runs - that means JShop installed, but for unification :))
		$JSInstalled = $this->_checkForJS();
		# Checking for VMart is installed
		$VMInstalled = $this->_checkForVM();
		
		# Building page header and controls
		JToolBarHelper::title(_JSHOP_IMPORT. ' "'.$this->ieTbl->get('name').'"', 'generic.png' );
		JToolBarHelper::custom("backtolistie", "back", 'browser.png',
							   _JSHOP_BACK_TO.' "'._JSHOP_PANEL_IMPORT_EXPORT.'"', false );

		# Don't show "Save" button if no VMart present
		if ($VMInstalled && $JSInstalled && count($this->vmConfig['active_languages'])) {
			JToolBarHelper::spacer();
			JToolBarHelper::save("save", _JSHOP_IMPORT);
		}
		else{
            if (!$VMInstalled || !$JSInstalled){
                JError::raiseWarning('', '* '._JSHOP_IE_VMIMPORT_NOTINSTALLED_WARNING);
            } else {
                JError::raiseWarning('', '* '._JSHOP_IE_VMIMPORT_SELECT_LANG_IN_VM);
            }
        }
			
		
		# Generate warning if JShop Data delete is checked
		if ($this->deleteJSData)
			JError::raiseNotice('', '* '._JSHOP_IE_VMIMPORT_DELETEJSDATA_WARNING);
		
		JError::raiseNotice('', '* '._JSHOP_IE_VMIMPORT_CURRENCIES_NOTICE);
		
		# Generate choise options
		$choiceOptions = array(
			JHTML::_('select.option', 1, _JSHOP_IE_VMIMPORT_YES),
			JHTML::_('select.option', 0, _JSHOP_IE_VMIMPORT_NO)
		);
		
		
		$JHTMLDeleteJSData = JHTMLSelect::radiolist($choiceOptions, 'delete_js_data', null, 'value', 'text',
													$this->deleteJSData, false);

		$JHTMLMakeSubproductsRelative = JHTMLSelect::radiolist($choiceOptions, 'make_subproducts_relative',
															   null, 'value', 'text',
															   $this->makeSubproductsRelative, false);

		$JHTMLAddAttrsToSubproductTitle = JHTMLSelect::radiolist($choiceOptions,
																 'add_attrs_to_subproduct_title',
																 null, 'value', 'text',
																 $this->addAttrsToSubproductTitle, false);
		
		$JHTMLMakeFreeAttributesRequired = JHTMLSelect::radiolist($choiceOptions,
																  'make_free_attributes_required', null,
																  'value', 'text',
																  $this->makeFreeAttributesRequired, false);
		
		# Generate attributes style options (jshop | vmart)
		$attrStyleOptions = array(
			JHTML::_('select.option', 'jshop', _JSHOP_IE_VMIMPORT_JSHOP),
			JHTML::_('select.option', 'vmart', _JSHOP_IE_VMIMPORT_VMART)
		);
		
		$JHTMLAttrStyle = JHTMLSelect::radiolist($attrStyleOptions, 'attr_style', null, 'value', 'text',
												 $this->attrStyle, false);
		
		
		$JHTMLCharacteristicPrefix = JHTMLSelect::radiolist($choiceOptions, 'characteristic_prefix', null,
															'value', 'text', $this->characteristicPrefix,
															false);
		
		# Generate image options (full | thumb)
		$imageOptions = array(
			JHTML::_('select.option', 'full', _JSHOP_IE_VMIMPORT_IMAGEFULL),
			JHTML::_('select.option', 'thumb', _JSHOP_IE_VMIMPORT_IMAGETHUMB)
		);
        $importLanguageOptions = array(JHTML::_('select.option', 'auto', _JSHOP_IE_VMIMPORT_AUTO));
		foreach ($this->vmConfig['active_languages'] as $language){
            $importLanguageOptions[] = JHTML::_('select.option', $language, JText::_($language));      
        }  

		$JHTMLCatImage = JHTMLSelect::radiolist($imageOptions, 'cat_image', null, 'value', 'text',
												$this->catImage, false);
		
        $JHTMLImportLanguage = JHTML::_('select.genericlist', $importLanguageOptions, $name = 'import_language', $attribs = null, $key = 'value', $text = 'text', $selected = NULL, $idtag = false, $translate = false );

		$JHTMLVendorLogo = JHTMLSelect::radiolist($imageOptions, 'vendor_logo', null, 'value', 'text',
												  $this->vendorLogo, false);
		
		$JHTMLResizeImages = JHTMLSelect::radiolist($choiceOptions, 'resize_images', null, 'value', 'text',
													$this->resizeImages, false);
		
		$JHTMLAutofillMeta = JHTMLSelect::radiolist($choiceOptions, 'autofill_meta', null, 'value', 'text',
													$this->autofillMeta, false);
		
		$JHTMLGenerateAliases = JHTMLSelect::radiolist($choiceOptions, 'generate_aliases', null, 'value',
													   'text', $this->generateAliases, false);								  
		# Show start template
		include (dirname(__FILE__).'/'.'startpage.php');
	}

	# Here we are after Import button click
	public function save() {
        header( 'Content-type: text/html; charset=utf-8' );
		if (isset($_POST['manual_execute'])) {
			# Manual execution
			
			# Don't work in constructor
			$this->loadLanguageFile();

			# Set settings
			$this->deleteJSData = JRequest::getVar('delete_js_data');
			
			$this->makeSubproductsRelative = JRequest::getVar('make_subproducts_relative');
			$this->addAttrsToSubproductTitle = JRequest::getVar('add_attrs_to_subproduct_title');
			$this->makeFreeAttributesRequired = JRequest::getVar('make_free_attributes_required');
			$this->attrStyle = JRequest::getVar('attr_style');
			$this->characteristicPrefix = JRequest::getVar('characteristic_prefix');
			
			$this->catImage = JRequest::getVar('cat_image');
            $this->importLanguage = JRequest::getVar('import_language'); 
			$this->vendorLogo = JRequest::getVar('vendor_logo');
			$this->resizeImages = JRequest::getVar('resize_images');
			
			$this->autofillMeta = JRequest::getVar('autofill_meta');
			$this->generateAliases = JRequest::getVar('generate_aliases');
			

			# Save settings to DB
			$params = parseParamsToArray($this->ieTbl->params); // There can be another params excepting settings
			
			$params['deleteJSData'] = $this->deleteJSData;
			
			$params['makeSubproductsRelative'] = $this->makeSubproductsRelative;
			$params['addAttrsToSubproductTitle'] = $this->addAttrsToSubproductTitle;
			$params['makeFreeAttributesRequired'] = $this->makeFreeAttributesRequired;
			$params['attrStyle'] = $this->attrStyle;
			$params['characteristicPrefix'] = $this->characteristicPrefix;
			
			$params['catImage'] = $this->catImage;
            $params['importLanguage'] = $this->importLanguage;  
			$params['vendorLogo'] = $this->vendorLogo;
			$params['resizeImages'] = $this->resizeImages;
			
			$params['autofillMeta'] = $this->autofillMeta;
			$params['generateAliases'] = $this->generateAliases;

			$this->ieTbl->params = parseArrayToParams($params);
			$this->ieTbl->store();
			
			# Output start message
			echo "<font style='font-size: 14px;'>";
			echo _JSHOP_IE_VMIMPORT_STARTED;
			echo "<br />";
			
			# Disable buffering, so we can see progress for each file download
			# (later in each download iteration I made flush)

			ob_end_flush();
			# Truncate tables if needed
			if ($this->deleteJSData) {
				echo '<br />'._JSHOP_IE_VMIMPORT_DELETINGJSDATA.'... ';
				flush();
				ob_flush();
				$this->_deleteJSData();
				echo '<font color="green">'._JSHOP_IE_VMIMPORT_SUCCEEDED.'</font></b><br><br>';
			}
			
			# Make import
			$this->_import();

			# Output finish message
			$redirectURL = JRoute::_('index.php?option=com_jshopping&controller=importexport&task=view&ie_id='.
									 $this->ieTbl->id);
			
			echo "<br /><br />";
			echo "<font style='font-size: 14px;'>";
			echo _JSHOP_IE_VMIMPORT_FINISHED;
			echo "<br /><br /><input type='button' onclick='location.href = \"$redirectURL\";'
				 value='"._JSHOP_IE_VMIMPORT_BACKTOIMPORT."' />";
			echo "</font>";
			
			exit;
		}
		else {
			# Automatic execution
			
			# Truncate tables if needed
			if ($this->deleteJSData)
				$this->_deleteJSData();
			
			$this->_import();
			
			# Save last execution time
			$this->ieTbl->set('endstart', time());
			$this->ieTbl->store();
		}
	}
}

?>